@@ -3,6 +3,9 @@
 #include "../IFinder.hpp"
 
 #include <hyprutils/os/FileDescriptor.hpp>
+#include <filesystem>
+
+#include <filesystem>
 
 class CDesktopEntry;
 class CEntryCache;
@@ -20,19 +23,19 @@ class CDesktopFinder : public IFinder {
     void                               onInotifyEvent();
 
   private:
-    std::vector<SP<CDesktopEntry>> m_desktopEntryCache;
-    std::vector<SP<IFinderResult>> m_desktopEntryCacheGeneric;
+    std::vector<SP<CDesktopEntry>>     m_desktopEntryCache;
+    std::vector<SP<IFinderResult>>     m_desktopEntryCacheGeneric;
 
-    std::vector<std::string>       m_desktopEntryPaths;
-    std::vector<int>               m_watches;
+    std::vector<std::filesystem::path> m_desktopEntryPaths;
+    std::vector<int>                   m_watches;
 
-    std::vector<std::string>       m_envPaths;
+    std::vector<std::filesystem::path> m_envPaths;
 
-    UP<CEntryCache>                m_entryFrequencyCache;
+    UP<CEntryCache>                    m_entryFrequencyCache;
 
-    void                           cacheEntry(const std::string& path);
-    void                           replantWatch();
-    void                           recache();
+    void                               cacheEntry(const std::filesystem::path& path);
+    void                               replantWatch();
+    void                               recache();
 
     friend class CDesktopEntry;
 };
