@@ -32,6 +32,10 @@ class CIPCEntry : public IFinderResult {
         return 0;
     }
 
+    virtual const std::string& name() {
+        return m_entry;
+    }
+
     virtual void run() {
         Debug::log(TRACE, "Selected {}", m_entry);
     }
@@ -51,7 +55,7 @@ void CIPCFinder::setData(const std::vector<const char*>& data) {
     for (const auto& s : data) {
         auto e = m_entries.emplace_back(makeShared<CIPCEntry>());
         m_entriesGeneric.emplace_back(e);
-        e->m_entry = s;
+        e->m_entry    = s;
         e->m_fuzzable = s;
         std::ranges::transform(e->m_fuzzable, e->m_fuzzable.begin(), ::tolower);
     }
@@ -63,7 +67,7 @@ void CIPCFinder::setData(const std::vector<std::string>& data) {
     for (const auto& s : data) {
         auto e = m_entries.emplace_back(makeShared<CIPCEntry>());
         m_entriesGeneric.emplace_back(e);
-        e->m_entry = s;
+        e->m_entry    = s;
         e->m_fuzzable = s;
         std::ranges::transform(e->m_fuzzable, e->m_fuzzable.begin(), ::tolower);
     }
